@@ -115,37 +115,38 @@ class Stmt:
         event = self.context.sigs["self"][self.stmt.value.func.id]
         expected_topics, topics = [], []
         expected_data, data = [], []
-        for pos, is_indexed in enumerate(event.indexed_list):
-            if is_indexed:
-                expected_topics.append(event.args[pos])
-                topics.append(self.stmt.value.args[pos])
+        with self.context.internal_memory_scope(id(self.stmt)):
+            for pos, is_indexed in enumerate(event.indexed_list):
+                if is_indexed:
+                    expected_topics.append(event.args[pos])
+                    topics.append(self.stmt.value.args[pos])
+                else:
+                    expected_data.append(event.args[pos])
+                    data.append(self.stmt.value.args[pos])
+            topics = pack_logging_topics(
+                event.event_id, topics, expected_topics, self.context, pos=getpos(self.stmt),
+            )
+            inargs, inargsize, inargsize_node, inarg_start = pack_logging_data(
+                expected_data, data, self.context, pos=getpos(self.stmt),
+            )
+
+            if inargsize_node is None:
+                sz = inargsize
             else:
-                expected_data.append(event.args[pos])
-                data.append(self.stmt.value.args[pos])
-        topics = pack_logging_topics(
-            event.event_id, topics, expected_topics, self.context, pos=getpos(self.stmt),
-        )
-        inargs, inargsize, inargsize_node, inarg_start = pack_logging_data(
-            expected_data, data, self.context, pos=getpos(self.stmt),
-        )
+                sz = ["mload", inargsize_node]
 
-        if inargsize_node is None:
-            sz = inargsize
-        else:
-            sz = ["mload", inargsize_node]
-
-        return LLLnode.from_list(
-            [
-                "seq",
-                inargs,
-                LLLnode.from_list(
-                    ["log" + str(len(topics)), inarg_start, sz] + topics,
-                    add_gas_estimate=inargsize * 10,
-                ),
-            ],
-            typ=None,
-            pos=getpos(self.stmt),
-        )
+            return LLLnode.from_list(
+                [
+                    "seq",
+                    inargs,
+                    LLLnode.from_list(
+                        ["log" + str(len(topics)), inarg_start, sz] + topics,
+                        add_gas_estimate=inargsize * 10,
+                    ),
+                ],
+                typ=None,
+                pos=getpos(self.stmt),
+            )
 
     def parse_Call(self):
         is_self_function = (
@@ -166,25 +167,26 @@ class Stmt:
         if isinstance(msg, vy_ast.Name) and msg.id == "UNREACHABLE":
             return LLLnode.from_list(["assert_unreachable", test_expr], typ=None, pos=getpos(msg))
 
-        reason_str = msg.s.strip()
-        sig_placeholder = self.context.new_placeholder(BaseType(32))
-        arg_placeholder = self.context.new_placeholder(BaseType(32))
-        reason_str_type = ByteArrayType(len(reason_str))
-        placeholder_bytes = Expr(msg, self.context).lll_node
-        method_id = fourbytes_to_int(keccak256(b"Error(string)")[:4])
-        assert_reason = [
-            "seq",
-            ["mstore", sig_placeholder, method_id],
-            ["mstore", arg_placeholder, 32],
-            placeholder_bytes,
-            [
-                "assert_reason",
-                test_expr,
-                int(sig_placeholder + 28),
-                int(4 + get_size_of_type(reason_str_type) * 32),
-            ],
-        ]
-        return LLLnode.from_list(assert_reason, typ=None, pos=getpos(self.stmt))
+        with self.context.internal_memory_scope(id(self.stmt)):
+            reason_str = msg.s.strip()
+            sig_placeholder = self.context.new_placeholder(BaseType(32))
+            arg_placeholder = self.context.new_placeholder(BaseType(32))
+            reason_str_type = ByteArrayType(len(reason_str))
+            placeholder_bytes = Expr(msg, self.context).lll_node
+            method_id = fourbytes_to_int(keccak256(b"Error(string)")[:4])
+            assert_reason = [
+                "seq",
+                ["mstore", sig_placeholder, method_id],
+                ["mstore", arg_placeholder, 32],
+                placeholder_bytes,
+                [
+                    "assert_reason",
+                    test_expr,
+                    int(sig_placeholder + 28),
+                    int(4 + get_size_of_type(reason_str_type) * 32),
+                ],
+            ]
+            return LLLnode.from_list(assert_reason, typ=None, pos=getpos(self.stmt))
 
     def parse_Assert(self):
         test_expr = Expr.parse_value_expr(self.stmt.test, self.context)
@@ -231,6 +233,10 @@ class Stmt:
         if not 0 < len(self.stmt.iter.args) < 3:
             return
 
+        # attempt to use the type specified by type checking, fall back to `int128`
+        # this is a stopgap solution to allow uint256 - it will be properly solved
+        # once we refactor `vyper.parser`
+        iter_typ = self.stmt.target.get("_type") or "int128"
         block_scope_id = id(self.stmt)
         with self.context.make_blockscope(block_scope_id):
             # Get arg0
@@ -240,15 +246,15 @@ class Stmt:
             # Type 1 for, e.g. for i in range(10): ...
             if num_of_args == 1:
                 arg0_val = self._get_range_const_value(arg0)
-                start = LLLnode.from_list(0, typ="int128", pos=getpos(self.stmt))
+                start = LLLnode.from_list(0, typ=iter_typ, pos=getpos(self.stmt))
                 rounds = arg0_val
 
             # Type 2 for, e.g. for i in range(100, 110): ...
             elif self._check_valid_range_constant(self.stmt.iter.args[1], raise_exception=False)[0]:
                 arg0_val = self._get_range_const_value(arg0)
                 arg1_val = self._get_range_const_value(self.stmt.iter.args[1])
-                start = LLLnode.from_list(arg0_val, typ="int128", pos=getpos(self.stmt))
-                rounds = LLLnode.from_list(arg1_val - arg0_val, typ="int128", pos=getpos(self.stmt))
+                start = LLLnode.from_list(arg0_val, typ=iter_typ, pos=getpos(self.stmt))
+                rounds = LLLnode.from_list(arg1_val - arg0_val, typ=iter_typ, pos=getpos(self.stmt))
 
             # Type 3 for, e.g. for i in range(x, x + 10): ...
             else:
@@ -261,7 +267,7 @@ class Stmt:
                 return
 
             varname = self.stmt.target.id
-            pos = self.context.new_variable(varname, BaseType("int128"), pos=getpos(self.stmt))
+            pos = self.context.new_variable(varname, BaseType(iter_typ), pos=getpos(self.stmt))
             self.context.forvars[varname] = True
             lll_node = LLLnode.from_list(
                 ["repeat", pos, start, rounds, parse_body(self.stmt.body, self.context)],
